#include "def.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dir.h>

#ifndef NULL
#define NULL 0L
#endif


copy(frname, toname)
char *frname, *toname;
{
	/* do this the dumb way (but faster on minix) -- bsd forked `cp' */
	register FILE *in, *out;
	register int c;

	if((in = fopen(frname, "r")) == (FILE *)NULL)
		return -1;
	if((out = fopen(toname,"w")) == (FILE *)NULL)
	{
		fclose(in);
		return -1;
	}
	while(((c = getc(in)) != EOF) && (!ferror(in)))
	{
		putc(c, out);
		if(ferror(out)) 
		{
			ewprintf("Write I/O error");
			fclose(in); fclose(out);
			return -1;
		}
	}
	if(ferror(in))
	{
		ewprintf("Read I/O error");
		fclose(in); fclose(out);
		return -1;
	}

	fclose(in); fclose(out);
	return 0;
}

#if 0 /* see below for a better suited version */
int rename(f1, f2)
char *f1, *f2;
{

	int status, pid, wpid;

	if ((pid = fork()) == 0) {
		execl("/usr/bin/mv", "mv", f1, f2, (char *)NULL);
		exit(0);
	}
	else if (pid > 0)
		while ((wpid = wait(&status))>=0 && wpid != pid)
			;
	else
	{
		ewprintf("Could not fork");
		return FALSE;
	}
	return status == 0;
}
#else
#ifndef __GNUC__	/* avail in gcc lib */
/* Quick and dirty rename  ++jrb bammi@dsrgsun.CES.cwru.edu             */
/* will only rename files, will not handle dirs                         */

#include <errno.h>

extern int errno;

#ifndef TEST
#define err(X) ewprintf(X)
#define std_err(X) ewprintf(X)
#else
#include <stdio.h>
#define std_err(X) fprintf(stderr, X)
#endif

/* rename old to new, return -1 on error, 0 on success */
int rename(old, new)
char   *old, *new;
{
    register int s1, s2;
    struct stat st1, st2;

    /* Don't do dirs or move a file onto itself. */
    if ( ((s1 = stat(old, &st1)) == 0) && ((s2 = stat(new, &st2)) == 0)
	&& (st1.st_dev == st2.st_dev) && (st1.st_ino == st2.st_ino) )
    {
	    err("can't move a file onto itself");
	    errno = EINVAL; /* strange?? */
	    return -1;
    }

    if(  ((s1==0) && ((st1.st_mode & S_IFMT) == S_IFDIR))
      || ((s2==0) && ((st2.st_mode & S_IFMT) == S_IFDIR)) )
    {
	err("we don'nt do directories");
	errno = EINVAL; /* strangeness again! */
	return -1;
    }

    if (!s2)
	    unlink (new); /* ignore any err here */

    if (link(old, new))
    { /* link failed, try a cp */
	if(copy(old, new)) return -1;
    }

    utime (new, &st1.st_atime);
    return unlink(old);

}

#ifdef TEST
err(msg)
char *msg;
{
	std_err("rename: ");
	std_err (msg);
	std_err ("\n");
	return;
}

#endif /* TEST */
#endif /* __GNUC__ */
#endif /* #if 0 #else */


#ifndef __GNUC__
/* getwd shamelessly adapted from:              ++jrb               */
/* pwd - print working directory		Author: Adri Koppes */

/* char *getwd(path)
 * char *path;
 * fill CWD in path, returns path
 * on error return NULL with error string in path
 */

char *getwd(path)
char *path;
{
	register int fd;
	register char *n;
	char name[128];
	extern char *rindex();
	struct stat s, st;
	struct direct d;

	*name = 0;
	stat(".", &s);
	do {
		if ((fd = open("..",0)) < 0) {
			strcpy(path, "Can't open ..");
			return (char *)NULL;
		}
		st.st_dev = s.st_dev;
		st.st_ino = s.st_ino;
		st.st_mode = s.st_mode;
		st.st_nlink = s.st_nlink;
		st.st_uid = s.st_uid;
		st.st_gid = s.st_gid;
		st.st_rdev = s.st_rdev;
		st.st_size = s.st_size;
		st.st_atime = s.st_atime;
		st.st_mtime = s.st_mtime;
		st.st_ctime = s.st_ctime;
		stat("..", &s);
		chdir("..");
		if (s.st_dev == st.st_dev)
			do
				if (read(fd, (char *)&d,
					 (int)sizeof(struct direct)) <
				    (int)sizeof(struct direct))
				{  
					strcpy(path,"Can't read ..");
					return (char *)NULL;
				}
			while(d.d_ino != st.st_ino);
		else
			do {
				if (read(fd, (char *)&d,
					 (int)sizeof(struct direct)) <
				    (int)sizeof(struct direct))
				{  
					strcpy(path, "Can't read ..");
					return (char *)NULL;
				}
				stat(d.d_name, &s);
			} while ((s.st_dev != st.st_dev) || (s.st_ino != st.st_ino));
		close(fd);
		if (strcmp(".",d.d_name)) {
			strcat(name,"/");
			strcat(name,d.d_name);
		}
	} while ((s.st_ino != st.st_ino) || (s.st_dev != st.st_dev));
	if (!*name)
	{
		strcpy(path, "/");
		return path;
	}
	*path = '\0';
	while (n = rindex(name, '/')) {
		strcat(path,n);
		*n = 0;
	}
	strcat(path, name);
	return path;
}
#endif /* __GNUC__ */

#ifndef NO_DIRED
int unlinkdir(f)
char *f;
{
	int status, pid, wpid;

	if ((pid = fork()) == 0) {
		execl("/usr/bin/rmdir", "rmdir", f, (char *)NULL);
		exit(0);
	}
	else if (pid > 0)
		while ((wpid = wait(&status))>=0 && wpid != pid)
			;
	else
	{
		ewprintf("Could not fork");
		return FALSE;
	}
	
	return status == 0;
}
#endif /* NO_DIRED */

/*  - eof - */
